/*
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <libxfce4util/libxfce4util.h>

#include "libappstarter/as.h"

#define PLUGIN "custom"

enum
{
    TARGET_URI_LIST,
    TARGET_UTF8_STRING,
    TARGET_TEXT,
    TARGET_COMPOUND_TEXT,
    TARGET_STRING,
    TARGET_TEXT_PLAIN,
    TARGET_MOZ_URL,
};

static const GtkTargetEntry targets[] =
{
    { "text/uri-list", 0, TARGET_URI_LIST },
    { "text/x-moz-url", 0, TARGET_MOZ_URL },
    { "UTF8_STRING", 0, TARGET_UTF8_STRING },
    { "TEXT", 0, TARGET_TEXT },
    { "COMPOUND_TEXT", 0, TARGET_COMPOUND_TEXT },
    { "STRING", 0, TARGET_STRING },
    { "text/plain", 0, TARGET_TEXT_PLAIN },
};

gchar *command = NULL;
gchar *conffile = NULL;
gboolean as_root = FALSE;

gchar *
get_cmd()
{
    gchar * ret;
    if (as_root)
        ret = g_strconcat ("sudo ", command, NULL);
    else
        ret = g_strdup (command);
    return ret;
}

static void
save_config ()
{
    XfceRc *rc = as_rc_open (conffile, FALSE);
    if (!rc)
        return;
    xfce_rc_set_group (rc, PLUGIN);
    xfce_rc_write_entry (rc, "command", command);
    xfce_rc_write_bool_entry (rc, "as_root", as_root);
    xfce_rc_close (rc);
}

static void
chk_toggled (GtkWidget *chk, gpointer data)
{
    as_root = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (chk));
    save_config ();
}

static void
entry_changed (GtkWidget *entry, gpointer data)
{
    g_free (command);
    command = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
    save_config ();
}

static void
drag_data (GtkWidget *widget, GdkDragContext *context,
           gint x, gint y, GtkSelectionData *selection_data,
           guint info, guint time)
{
    gchar *text;
    switch (info)
    {
        case TARGET_STRING:
        case TARGET_UTF8_STRING:
        case TARGET_COMPOUND_TEXT:
        case TARGET_TEXT:
            text = (gchar *) gtk_selection_data_get_text (selection_data);
            if (G_LIKELY (text != NULL))
            {
                if (G_LIKELY (*text != '\0'))
                    gtk_entry_set_text (GTK_ENTRY (widget), text);
                g_free (text);
            }
            break;

        case TARGET_TEXT_PLAIN:
            gtk_entry_set_text (GTK_ENTRY (widget),
                (const gchar *) selection_data->data);
            break;

        case TARGET_URI_LIST:
            text = g_strndup ((const gchar *) selection_data->data,
                selection_data->length);
            gchar **uris = g_uri_list_extract_uris (text);
            g_free (text);
            /* we use the first element only */
            gchar *filename = g_filename_from_uri (uris[0], NULL, NULL);
            gtk_entry_set_text (GTK_ENTRY (widget), filename);
            g_strfreev (uris);
        break;
    }
}

static void
read_config()
{
    XfceRc *rc;
    rc = as_rc_open (conffile, TRUE);
    if (!rc)
        return;

    xfce_rc_set_group(rc, PLUGIN);
    g_free (command);
    command = g_strdup (xfce_rc_read_entry (rc, "command", ""));
    as_root = xfce_rc_read_bool_entry (rc, "as_root", FALSE);

    xfce_rc_close(rc);
}

GtkWidget *
get_config_widget(gchar *file)
{
    conffile = g_strdup (file);
    GtkWidget *label, *hbox, *vbox;
    GtkWidget *entry, *root_chk;

    vbox = gtk_vbox_new (FALSE, 0);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

    label = gtk_label_new (_("Enter a command: "));
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);

    read_config ();

    entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (entry), command);
    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 2);
    gtk_drag_dest_set (entry, GTK_DEST_DEFAULT_ALL,
                       targets, G_N_ELEMENTS (targets),
                       GDK_ACTION_COPY | GDK_ACTION_LINK);

    root_chk = gtk_check_button_new_with_label (_("Run as root"));
    gtk_box_pack_start (GTK_BOX (vbox), root_chk, FALSE, FALSE, 2);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (root_chk), as_root);
    gtk_widget_show (root_chk);

    g_signal_connect (G_OBJECT (entry), "drag_data_received",
        G_CALLBACK (drag_data), NULL);
    g_signal_connect (G_OBJECT (entry), "changed",
        G_CALLBACK (entry_changed), NULL);
    g_signal_connect (G_OBJECT (root_chk), "toggled",
        G_CALLBACK (chk_toggled), NULL);

    gtk_widget_show_all (hbox);
    return vbox;
}

void
free_all()
{
    g_free (command);
    g_free (conffile);
}
